"""
Tests for conveyor.retieve module
"""
from pathlib import Path
from unittest.mock import patch

from conveyor.retrieve import RetrievalConveyor

settings = {
    "qa_delivery_area": "/lustre/aoc/cluster/pipeline/docker/workspaces/qa2",
    "weblog_cache_area": "/lustre/aoc/cluster/pipeline/docker/workspaces/cache/weblog",
    "workspaces_lustre_root_dir": "/lustre/aoc/cluster/pipeline/docker/workspaces/spool",
    "current_root_directory": "current/directory",
    "current_subdirectory": "dir_123",
    "destination_dir": "/lustre/aoc/cluster/pipeline/docker/workspaces/spool/tmpabcd1234",
    "destination_subdir": "tmpabcd1234",
}

action = "Calibration Retrieval"

conveyor = RetrievalConveyor(settings=settings, action=action)


def fake_determine_qa():
    qa_area = settings["qa_delivery_area"]
    wf_dir = settings["destination_subdir"]
    return Path(qa_area + "/" + wf_dir)


def fake_determine_spool():
    spool_area = settings["workspaces_lustre_root_dir"]
    wf_dir = settings["destination_subdir"]
    return Path(spool_area + "/" + wf_dir)


class TestRetrievalConveyor:
    @patch("pathlib.Path.rmdir")
    @patch("glob.glob")
    @patch("os.chdir")
    def test_retrieval(self, mock_chdir, mock_glob, mock_rm):
        with patch("conveyor.retrieve.RetrievalConveyor.break_symlinks") as symlinks:
            with patch("conveyor.retrieve.RetrievalConveyor.move_subdirectories_to_spool") as move:
                with patch("conveyor.retrieve.RetrievalConveyor.check_spool_contents") as contents:
                    conveyor.retrieval()
                    assert mock_chdir.call_count == 1
                    assert mock_glob.call_count == 1
                    assert symlinks.call_count == 1

    @patch("pathlib.Path.is_symlink", return_value=True)
    @patch("pathlib.Path.unlink")
    def test_break_symlinks(self, mock_unlink, mock_sym):
        path = fake_determine_spool()
        dir_list = ["working/", "rawdata/", "products/"]
        conveyor.break_symlinks(path, dir_list)
        assert mock_sym.call_count == 3
        assert mock_unlink.call_count == 3

    @patch("shutil.move")
    def test_move_subdirectories_to_spool(self, mock_move):
        qa_path = fake_determine_qa()
        spool_path = fake_determine_spool()
        dir_list = ["working/", "rawdata/", "products/"]
        conveyor.move_subdirectories_to_spool(qa_path, spool_path, dir_list)
        assert mock_move.call_count == 3

    def test_determine_qa_directory(self):
        path = conveyor.determine_qa_directory()
        assert isinstance(path, Path)

    def test_determine_spool_directory(self):
        path = conveyor.determine_spool_directory()
        assert isinstance(path, Path)

    @patch("glob.glob", return_value=["working/", "rawdata/", "products/"])
    @patch("os.chdir")
    def test_check_spool_contents(self, mock_chdir, mock_glob):
        spool = fake_determine_spool()
        expected_list = ["working/", "rawdata/", "products/"]
        check = conveyor.check_spool_contents(spool, expected_list)
        assert check is True

    @patch("glob.glob", return_value=["working/", "products/"])
    @patch("os.chdir")
    def test_check_spool_contents(self, mock_chdir, mock_glob):
        spool = fake_determine_spool()
        expected_list = ["working/", "rawdata/", "products/"]
        check = conveyor.check_spool_contents(spool, expected_list)
        assert check is False

    def test_convey(self):
        with patch("conveyor.retrieve.RetrievalConveyor.retrieval") as retrieval:
            conveyor.convey()
            assert retrieval.call_count == 1
