""" Pieces of any ingestion manifest """

import abc
import re
from enum import Enum
from pathlib import Path
from typing import Union, List, Dict

from ingest_envoy.utilities import ScienceProductType, Telescope, AncillaryProductType

MANIFEST_FILENAME = "ingestion_manifest.json"
ARTIFACT_NAME = "ingestion_artifacts_"
TARFILE_EXT = ".tar"
WEBLOG_FILENAME = "weblog.tgz"
SCIENCE_PRODUCT_PATTERN = re.compile("[a-zA-Z0-9._\\-+]*\\.tar")
JSON = Union[int, float, str, List["JSON"], Dict[str, "JSON"]]


class IngestionManifestKey(Enum):
    """Sections we expect to see in a manifest"""

    PARAMETERS = "parameters"
    INPUT_GROUP = "input_group"
    OUTPUT_GROUP = "output_group"
    INGESTION_PATH = "ingestion_path"
    SCIENCE_PRODUCTS = "science_products"
    ANCILLARY_PRODUCTS = "ancillary_products"


class ParamsKey(Enum):
    """Keys in ManifestParameters"""

    PARAMETERS = "parameters"
    TELESCOPE = "telescope"
    REINGEST = "reingest"
    NGAS_INGEST = "ngas_ingest"
    CALIBRATE = "calibrate"
    INGESTION_PATH = IngestionManifestKey.INGESTION_PATH.value


class ManifestComponentIF(abc.ABC):
    """Interface for components of an ingestion manifest (including the manifest itself)"""

    @abc.abstractmethod
    def __eq__(self, other):
        return NotImplemented

    @abc.abstractmethod
    def to_json(self) -> JSON:
        """
        JSON-ify this object

        :return: json.load()-able string
        """
        return NotImplemented


class InputScienceProduct(ManifestComponentIF):
    """Simplest type of science product: has only a locator"""

    def __init__(self, locator: str):
        self.locator = locator

    def __eq__(self, other):
        if isinstance(other, InputScienceProduct):
            return other.locator == self.locator

    def to_json(self) -> JSON:
        """
        Turn me into a json-ifiable dict

        :return: dicty-me
        """
        return {"locator": self.locator}


class InputGroup(ManifestComponentIF):
    """Generic ingestion manifest input group"""

    def __init__(self, science_products: List[InputScienceProduct]):
        self.science_products = science_products

    def __eq__(self, other):
        if isinstance(other, InputGroup):
            return other.science_products == self.science_products

        return False

    def to_json(self) -> JSON:
        """
        Turn me into a json-ifiable dict

        :return: dicty-me
        """
        me_dict = dict(self.__dict__)

        sps = me_dict[IngestionManifestKey.SCIENCE_PRODUCTS.value]
        sps = [sp.to_json() for sp in sps]
        if len(sps) == 0:
            return {}

        to_return = {IngestionManifestKey.SCIENCE_PRODUCTS.value: sps}
        return to_return


class ManifestParameters(ManifestComponentIF):
    """Represents "parameters" section of ingestion manifest"""

    def __init__(
        self,
        telescope: Telescope,
        reingest: bool,
        ngas_ingest: bool,
        calibrate: bool,
        staging_source_dir: Path,
    ):
        self.telescope = telescope
        self.reingest = reingest
        self.ngas_ingest = ngas_ingest
        self.calibrate = calibrate
        self.staging_source_dir = staging_source_dir

    def __eq__(self, other):
        if isinstance(other, ManifestParameters):
            return (
                other.telescope.value == self.telescope.value
                and other.reingest == self.reingest
                and other.ngas_ingest == self.ngas_ingest
                and other.calibrate == self.calibrate
                and other.staging_source_dir == self.staging_source_dir
            )

        return False

    def to_json(self) -> JSON:
        return {
            ParamsKey.TELESCOPE.value: self.telescope,
            ParamsKey.REINGEST.value: str(self.reingest).lower(),
            ParamsKey.NGAS_INGEST.value: str(self.ngas_ingest).lower(),
            ParamsKey.CALIBRATE.value: str(self.calibrate).lower(),
            ParamsKey.INGESTION_PATH.value: str(self.staging_source_dir),
        }


class OutputScienceProduct(ManifestComponentIF):
    """Generic science product contained in manifest output group"""

    def __init__(self, type: ScienceProductType, filename: str):
        self.type = type
        self.filename = filename

    def __eq__(self, other):
        if isinstance(other, OutputScienceProduct):
            return other.type == self.type and other.filename == self.filename

        return False

    def to_json(self) -> JSON:
        return {"type": self.type.value, "filename": self.filename}


class AncillaryProduct(ManifestComponentIF):
    """Represents an ancillary product in an ingestion manifest"""

    def __init__(
        self,
        type: AncillaryProductType,
        filename: str,
        science_associate: str = None,
        group_with: str = None,
    ):
        self.type = type
        self.filename = filename
        self.science_associate = science_associate
        self.group_with = group_with

        # make this an ancillary to a particular science product (assumes locator string)
        if science_associate:
            self.science_associate = science_associate  # TODO, possibly: enum?
        # make this an ancillary to the group of a science product (assumes locator string)
        if group_with:
            self.group_with = group_with

    def __eq__(self, other):
        if isinstance(other, AncillaryProduct):
            return (
                other.type == self.type
                and other.filename == self.filename
                and other.group_with == self.group_with
                and other.science_associate == self.science_associate
            )

        return False

    def to_json(self) -> JSON:
        """
        Turn me into a json-ifiable dict

        :return: dicty-me
        """
        me_dict = dict(self.__dict__)
        me_dict["type"] = self.type.value
        clean_dict = {}
        for key, val in me_dict.items():
            if val is not None:
                clean_dict[key] = val

        return clean_dict


class OutputGroup(ManifestComponentIF):
    """Generic ingestion manifest output group"""

    def __init__(
        self,
        science_products: List[OutputScienceProduct],
        ancillary_products: List[AncillaryProduct] = None,
    ):
        self.science_products = science_products
        self.ancillary_products = ancillary_products

    def __eq__(self, other):
        if isinstance(other, OutputGroup):
            return (
                other.science_products == self.science_products
                and other.ancillary_products == self.ancillary_products
            )

        return False

    def to_json(self) -> JSON:
        """
        Turn me into a json-ifiable dict

        :return: dicty-me
        """
        me_dict = dict(self.__dict__)

        sps = me_dict[IngestionManifestKey.SCIENCE_PRODUCTS.value]
        sps = [sp.to_json() for sp in sps]
        me_dict[IngestionManifestKey.SCIENCE_PRODUCTS.value] = sps

        aps = me_dict[IngestionManifestKey.ANCILLARY_PRODUCTS.value]
        if aps:
            aps = [ap.to_json() for ap in aps]
            me_dict[IngestionManifestKey.ANCILLARY_PRODUCTS.value] = aps

        return me_dict
        # return {IngestionManifestKey.OUTPUT_GROUP.value: me_dict}


class Weblog:
    """Represents a weblog.tgz as an ancillary product"""

    def __init__(self, weblog_path: Path):
        self.ancillary_product = {"type": "weblog", "filename": str(weblog_path)}

    def to_json(self) -> JSON:
        """
        JSON-ify this object

        :return: json.load()-able string
        """
        return dict(self.__dict__)
